import {
  Expression,
  NumberExpression,
  NumberExpressionConverter,
  StringExpression,
  StringExpressionConverter,
} from "adaptive-expressions";
import { Count } from "adaptive-expressions/lib/builtinFunctions";

import {
  Converter,
  ConverterFactory,
  Dialog,
  DialogConfiguration,
  DialogContext,
  DialogTurnResult,
} from "botbuilder-dialogs";

import { MicrosoftAppCredentials } from "botframework-connector";
import { AttachmentActions } from "./actions/attachmentActions";

export interface BotFrameworkAttachmentCustomActionsConfiguration extends DialogConfiguration {
  url: string | string | Expression | StringExpression;
  action: string | string | Expression | StringExpression;
  channelId: string | string | Expression | StringExpression;
  msAppId: string | string | Expression | StringExpression;
  msAppPwd: string | string | Expression | StringExpression;
  resultProperty?: string | Expression | StringExpression;
}

export class BotFrameworkAttachmentCustomActions
  extends Dialog
  implements BotFrameworkAttachmentCustomActionsConfiguration {
  public static $kind = "BotbuilderSamples.BotFrameworkAttachmentCustomActions";

  public url: StringExpression = new StringExpression('');
  public action: StringExpression = new StringExpression('');
  public channelId: StringExpression = new StringExpression('');
  public msAppId: StringExpression = new StringExpression('');
  public msAppPwd: StringExpression = new StringExpression('');
  public resultProperty?: StringExpression;

  public getConverter(
    property: keyof BotFrameworkAttachmentCustomActionsConfiguration
  ): Converter | ConverterFactory {
    switch (property) {
      case "url":
        return new StringExpressionConverter();
      case "action":
        return new StringExpressionConverter();
      case "channelId":
        return new StringExpressionConverter();
      case "msAppId":
        return new StringExpressionConverter();
      case "msAppPwd":
        return new StringExpressionConverter();
      case "resultProperty":
        return new StringExpressionConverter();
      default:
        return super.getConverter(property);
    }
  }

  public async beginDialog(dc: DialogContext): Promise<DialogTurnResult> {
    const action = this.action.getValue(dc.state);
    let result;

    if (action === 'GET_BASE64_CONTENT') {
      let token: string = '';
      const channelId = this.channelId.getValue(dc.state);
      if (channelId == 'msteams' || channelId == 'skype') {
        const credentials = new MicrosoftAppCredentials(this.msAppId.getValue(dc.state), this.msAppPwd.getValue(dc.state));
        //const credentials = new MicrosoftAppCredentials('32fc951d-383f-485b-be57-a71480ec6683', 'Yc67Q~~QaC39AMKrmDd6XUo54bOY5ukbbqCKV');
        token = await credentials.getToken(false);
      }
      result = await AttachmentActions.getBase64Content(this.url.getValue(dc.state), token);
      
      // // var contentBytes = new List<byte[]>();
      // using (var connectorClient = new ConnectorClient(new Uri(activity.ServiceUrl)))
      // {
      //     var token = await(connectorClient.Credentials as MicrosoftAppCredentials).GetTokenAsync();
      //     foreach (var content in attachments)
      //     {
      //         var uri = new Uri(content.Item2);
      //         using (var httpClient = new HttpClient())
      //         {
               
      //                 httpClient.DefaultRequestHeaders.Authorization = new AuthenticationHeaderValue("Bearer", token);
      //                 httpClient.DefaultRequestHeaders.Accept.Add(new MediaTypeWithQualityHeaderValue("application/octet-stream"));
                
      //             contentBytes.Add(await httpClient.GetByteArrayAsync(uri));
      //         }
      //     }
      // }
      // return contentBytes;
    }

    if (this.resultProperty) {
      dc.state.setValue(this.resultProperty.getValue(dc.state), result);
    }
    return dc.endDialog(result);
  }

  protected onComputeId(): string {
    return "BotbuilderSamples.BotFrameworkAttachmentCustomActions";
  }
}
